﻿using System;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace GoogleMapsLayerTest8677
{
    public class CustomGoogleMap : GoogleMapsLayer
    {
        public CustomGoogleMap(string cacheDirectory, string clientId, string privateKey) : base(cacheDirectory, clientId, privateKey)
        { }

        protected override void OnCreatingRequest(ThinkGeo.MapSuite.Core.CreatingRequestGoogleMapsLayerEventArgs e)
        {
            e.RequestUri = new Uri(e.RequestUri.AbsoluteUri + "&key=xxxxxxxxx");
            base.OnCreatingRequest(e);
        }
    }

    public partial class Form1 : Form
    {
        string latestUri;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.Meter;
            winformsMap1.CurrentExtent = new RectangleShape(-20037508, 20037508, 20037508, -20037508);

            Application.ThreadException += Application_ThreadException;
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(MyHandler);

            CustomGoogleMap googleMapsLayer = new CustomGoogleMap(null, "", "") { MapType = GoogleMapsMapType.Satellite };
            //googleMapsLayer.WebProxy = new WebProxy("127.0.0.1", 1080);
            googleMapsLayer.TileMode = GoogleMapsTileMode.SingleTile;
            googleMapsLayer.SendingWebRequest += GoogleMapsLayer_SendingWebRequest;

            var layerOverlay = new LayerOverlay();
            layerOverlay.Layers.Add(googleMapsLayer);
            
            winformsMap1.Overlays.Add(layerOverlay);
            winformsMap1.Refresh();
        }

        private void GoogleMapsLayer_SendingWebRequest(object sender, SendingWebRequestEventArgs e)
        {
            lock (string.Intern("locker"))
            {
                latestUri = e.WebRequest.RequestUri.AbsoluteUri;

                string file = "log.txt";
                if (!File.Exists(file)) File.Create(file).Close();
                File.AppendAllLines(file, new[] { $"{DateTime.Now} | {latestUri}" });
            }
        }

        private void MyHandler(object sender, UnhandledExceptionEventArgs e)
        {
            lock (string.Intern("locker2"))
            {
                string file = "UnhandledExceptionLog.txt";
                if (!File.Exists(file)) File.Create(file).Close();
                string str = GetExceptionMsg(e.ExceptionObject as Exception, e.ToString());
                File.AppendAllLines(file, new[] { $"{DateTime.Now} | {latestUri} | { str }" });
            }
        }

        private void Application_ThreadException(object sender, ThreadExceptionEventArgs e)
        {
            lock (string.Intern("locker3"))
            {
                string file = "UnhandledExceptionLog.txt";
                if (!File.Exists(file)) File.Create(file).Close();
                string str = GetExceptionMsg(e.Exception, e.ToString());
                File.AppendAllLines(file, new[] { $"{DateTime.Now} | {latestUri} | { str }" });
            }
        }

        static string GetExceptionMsg(Exception ex, string backStr)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine("****************************Exception Text****************************");
            sb.AppendLine("【StartTime】：" + DateTime.Now.ToString());
            if (ex != null)
            {
                sb.AppendLine("【ExceptionType】：" + ex.GetType().Name);
                sb.AppendLine("【ExceptionMessage】：" + ex.Message);
                sb.AppendLine("【ExceptionStackTrace】：" + ex.StackTrace);
            }
            else
            {
                sb.AppendLine("【UnhandledException】：" + backStr);
            }
            sb.AppendLine("***************************************************************");
            return sb.ToString();
        }
    }
}
